--***************************************************************
--* File         : WBIPRT.vhd                                   *
--* Created      : 2007.02.26 kawasan                           *
--* Modified     : Time-stamp: <08/05/15 16:58:19 kawasan>      *
--*                                                             *
--* WISHBONE Parallel-Output-Port Controller                    *
--*                                                             *
--* Copyright 2007-2008 Osamu Kawashima. all rights reserved    *
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;

entity WBOPRT is
	generic (
		DATA_SIZE       : integer := 32
	);
	port (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        : in    std_logic;
		WB_RST_I        : in    std_logic;
		WB_DAT_I        : in    std_logic_vector(31 downto 0);
		WB_WE_I         : in    std_logic;
		WB_STB_I        : in    std_logic;
		WB_ACK_O        : out   std_logic;
		WB_DAT_O        : out   std_logic_vector(31 downto 0);

		-- Output port (non-WISHBONE signals):
		PRT_O           : out   std_logic_vector(DATA_SIZE-1 downto 0)
	);
end WBOPRT;

architecture RTL of WBOPRT is

	constant ZERO32	        : std_logic_vector(31 downto 0) := (others=>'0');

	signal	dat             : std_logic_vector(DATA_SIZE-1 downto 0);

begin

	WB_ACK_O <= WB_STB_I;

	process(WB_CLK_I)
	begin
		if WB_CLK_I'event and WB_CLK_I='1' then
			if WB_RST_I='1' then
				dat <= (others=>'0');
			elsif WB_STB_I='1' and WB_WE_I='1' then
				dat <= WB_DAT_I(dat'range);
			else
				dat <= dat;
			end if;
		end if;
	end process;

	uPRT_32BIT:
	if DATA_SIZE=32 generate
		WB_DAT_O<= dat;
	end generate;

	uPRT_OTHER:
	if DATA_SIZE/=32 generate
		WB_DAT_O<= ZERO32(31 downto dat'left+1) & dat(dat'range);
	end generate;

	PRT_O <= dat;

end RTL;
